#!/bin/bash

SOURCE_DIR=$(realpath $(dirname "${0}"))

# Install dependencies for Sahana
apt-get -y --no-install-recommends install python-dateutil python-imaging python-lxml python-matplotlib python-psycopg2 python-reportlab python-requests python-shapely python-xlrd python-xlwt

# Install web2py
wget http://web2py.com/examples/static/web2py_src.zip -O /srv/web2py.zip
unzip -d /srv /srv/web2py.zip
mv /srv/web2py /srv/sahana
rm -f /srv/web2py.zip

# Install Sahana
wget https://github.com/sahana/eden/archive/master.tar.gz -O /srv/sahana/applications/eden.tgz
tar xzf /srv/sahana/applications/eden.tgz -C /srv/sahana/applications/
mv /srv/sahana/applications/eden-master /srv/sahana/applications/eden
rm -f /srv/sahana/applications/eden.tgz

# Create PostgreSQL user and database
export SAHANA_PWD=$(head -c 260 /dev/urandom | tr -cd '[:alnum:]' | head -c 26)
envsubst <${SOURCE_DIR}/sahana/tmp/createdb-sahana.sql >/tmp/createdb-sahana.sql
sudo -u postgres psql -f /tmp/createdb-sahana.sql
rm -f /tmp/createdb-sahana.sql

# Configure web2py
cp ${SOURCE_DIR}/sahana/srv/sahana/routes.py /srv/sahana/routes.py
cp ${SOURCE_DIR}/sahana/srv/sahana/run_scheduler.py /srv/sahana/run_scheduler.py
ln -s /srv/sahana/handlers/wsgihandler.py /srv/sahana/wsgihandler.py

# Configure Sahana
mkdir /srv/sahana/applications/eden/{databases,errors,sessions}
mkdir -p /srv/sahana/applications/eden/static/cache/chart
mkdir -p /srv/sahana/applications/eden/uploads/{gis_cache,images,tracks}
export SAHANA_HMAC=$(head -c 260 /dev/urandom | tr -cd '[:alnum:]' | head -c 26)
envsubst <${SOURCE_DIR}/sahana/srv/sahana/applications/eden/models/000_config.py >/srv/sahana/applications/eden/models/000_config.py

# Create OS user
adduser --system --group --home /srv/sahana --shell /bin/false sahana
chown -R sahana:sahana /srv/sahana

# Populate database
cd /srv/sahana
sudo -u sahana python web2py.py -S eden -M -R applications/eden/static/scripts/tools/noop.py

# Set "production values" (increases performance)
sed -i 's/settings.base.migrate = True/settings.base.migrate = False/' /srv/sahana/applications/eden/models/000_config.py
sed -i 's/# settings.base.prepopulate = 0/settings.base.prepopulate = 0/' /srv/sahana/applications/eden/models/000_config.py
sudo -u sahana python web2py.py -S eden -M -R applications/eden/static/scripts/tools/compile.py

# Create uwsgi and nginx app definition
cp ${SOURCE_DIR}/sahana/etc/uwsgi/apps-available/sahana.ini /etc/uwsgi/apps-available/sahana.ini
ln -s /etc/uwsgi/apps-available/sahana.ini /etc/uwsgi/apps-enabled/sahana.ini
cp ${SOURCE_DIR}/sahana/etc/nginx/apps-available/sahana /etc/nginx/apps-available/sahana
ln -s /etc/nginx/apps-available/sahana /etc/nginx/apps-enabled/sahana

# Restart services
systemctl restart uwsgi
systemctl restart nginx

# TODO: Set cgroups

# TODO: Does matplotlib still require MPLCONFIGDIR directory?
# If so, /srv/sahana/wsgihandler.py needs to be modified

# TODO: Potentially needed dependencies
# WARNING:  Setup unresolved dependency: boto required for Setup Module
# WARNING:  S3Doc unresolved dependency: pyth required for RTF document support in Full-Text Search
# WARNING:  S3Doc unresolved dependency: sunburnt required for Full-Text Search support
# WARNING:  S3GIS unresolved dependency: selenium required for Map printing support
# WARNING:  Setup unresolved dependency: PyYAML required for Setup Module
# WARNING:  S3GIS unresolved dependency: GDAL required for Shapefile support
# WARNING:  Setup unresolved dependency: ansible required for Setup Module
# WARNING:  Survey unresolved dependency: PyRTF required if you want to export assessment/survey templates as a Word document
# WARNING: Unresolved optional dependency: pyserial required
# WARNING:  S3Msg unresolved dependency tweepy required for non-Tropo Twitter support
# WARNING:  Message Parsing unresolved dependency: TwitterSearch required for fetching results from twitter keyword queries
